
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module cocoa.appkit.nibdeclarations;

import objc.bridge;


private template ToUpper(char c) {
	static if (c >= 'a' && c <= 'z') {
		const char ToUpper = c - 'a' + 'A';
	} else {
		const char ToUpper = c;
	}
}


private template ToSetterSelector(char[] name) {
	const char[] ToSetterSelector = "set"~ ToUpper!(name[0]) ~ name[1..$] ~":";
}

static assert(ToSetterSelector!("test") == "setTest:");
static assert(ToSetterSelector!("HTMLValue") == "setHTMLValue:");
static assert(ToSetterSelector!("z") == "setZ:");


template IBAction(alias action, char[] name = (&action).stringof[2..$]) {
	mixin ObjcBindMethod!(action, void, name ~":", Object);
}

template IBOutlet(alias outlet, char[] name) {
	static assert(is(typeof(outlet) : Object), "IB outlets only work with object members.");
	// do we need getOutlet too?
	private final void setOutlet(typeof(outlet) value) {
		outlet = value;
	}
	pragma(msg, ToSetterSelector!(name));
	mixin ObjcBindMethod!(setOutlet, void, ToSetterSelector!(name), typeof(outlet));
}


private class UnitTest : ObjcObject {
	Object var;
	void act(Object sender) { }
	
	static assert((&act).stringof[2..$] == "act", 
		"stringof incomaptiblity ("~ (&act).stringof[2..$] ~").");
	
	mixin IBOutlet!(var, "var");
	mixin IBAction!(act);
	
}
