
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module cocoa.appkit.window;

import cocoa.appkit.graphics;

import cocoa.foundation.object;
import cocoa.foundation.string;
import cocoa.foundation.geometry;

import objc.bridge;
import objc.runtime;


enum {
	NSBorderlessWindowMask = 0,
	NSTitledWindowMask = 1 << 0,
	NSClosableWindowMask = 1 << 1,
	NSMiniaturizableWindowMask = 1 << 2,
	NSResizableWindowMask = 1 << 3
}

version (MAC_OS_X_VERSION_10_2) {

	/** Specifies a window with textured background (eg. metal) */
	enum {
		NSTexturedBackgroundWindowMask	= 1 << 8
	};

}

version (MAC_OS_X_VERSION_10_4) {

	/** Specifies a window that ignores the userSpaceScaleFactor of the 
		NSScreen on which it is created.  Currently restricted to borderless 
		windows (NSBorderlessWindowMask) */
	enum {
		NSUnscaledWindowMask		= 1 << 11
	};

	/** Specifies a window whose titlebar and toolbar have a unified look - 
		that is, a continuous background */
	enum {
		NSUnifiedTitleAndToolbarWindowMask	= 1 << 12
	};

}

/** used with NSRunLoop's performSelector:target:argument:order:modes: */
enum {
    NSDisplayWindowRunLoopOrdering	= 600000,
    NSResetCursorRectsRunLoopOrdering	= 700000
};

//#define NSNormalWindowLevel              kCGNormalWindowLevel
//#define NSFloatingWindowLevel		 kCGFloatingWindowLevel
//#define NSSubmenuWindowLevel		 kCGTornOffMenuWindowLevel
//#define NSTornOffMenuWindowLevel	 kCGTornOffMenuWindowLevel
//#define NSMainMenuWindowLevel		 kCGMainMenuWindowLevel
//#define NSStatusWindowLevel		 kCGStatusWindowLevel
//// NSDockWindowLevel is deprecated
//#define NSDockWindowLevel		 kCGDockWindowLevel
//#define NSModalPanelWindowLevel          kCGModalPanelWindowLevel
//#define NSPopUpMenuWindowLevel           kCGPopUpMenuWindowLevel
//#define NSScreenSaverWindowLevel         kCGScreenSaverWindowLevel


enum NSSelectionDirection {
    NSDirectSelection = 0,
    NSSelectingNext,
    NSSelectingPrevious
}

version (MAC_OS_X_VERSION_10_2) {
	// standard window buttons
	enum {
		NSWindowCloseButton,
		NSWindowMiniaturizeButton,
		NSWindowZoomButton,
		NSWindowToolbarButton,
		NSWindowDocumentIconButton
	}
}


class NSWindow : NSObject {
	mixin ObjcWrap;
	this(objc.runtime.id self) { super(self); }
	
	this(NSRect contentRect, uint styleMask, NSBackingStoreType backing, bit defer) {
		super(objcAlloc);
		objcInstance = invokeObjcSuper!(objc.runtime.id, "initWithContentRect:styleMask:backing:defer:")(contentRect, styleMask, backing, defer);
	}


	/** Get the title of this window. */
	NSString title() {
		return invokeObjcSuper!(NSString, "title")();
	}
	
	/** Set the title of this window. */
	void title(NSString str) {
		invokeObjcSuper!(void, "setTitle:")(str);
	}
	
	/** ditto */
	final void title(char[] str) {
		title(str.toNSString());
	}
	
	mixin ObjcBindMethod!(title, NSString, "title");
	mixin ObjcBindMethod!(title, void, "setTitle:", NSString);
	

	void center() {
		invokeObjcSuper!(void, "center")();
	}

	void makeKeyAndOrderFront(Object sender) {
		invokeObjcSuper!(void, "makeKeyAndOrderFront:")(sender);
	}
	
	mixin ObjcBindMethod!(center, void, "center");
	mixin ObjcBindMethod!(makeKeyAndOrderFront, void, "makeKeyAndOrderFront:", Object);
	
}
