
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module cocoa.foundation.geometry;

import objc.runtime;


struct _NSPoint {
    float x;
    float y;
}
alias _NSPoint NSPoint;

alias NSPoint *NSPointPointer;
alias NSPoint *NSPointArray;


struct _NSSize {
    float width;		/* should never be negative */
    float height;		/* should never be negative */
}
alias _NSSize NSSize;

alias NSSize *NSSizePointer;
alias NSSize *NSSizeArray;


struct _NSRect {
    NSPoint origin;
    NSSize size;
}
alias _NSRect NSRect;

alias NSRect *NSRectPointer;
alias NSRect *NSRectArray;


enum _NSRectEdge {
    NSMinXEdge = 0,
    NSMinYEdge = 1,
    NSMaxXEdge = 2,
    NSMaxYEdge = 3
}
alias _NSRectEdge NSRectEdge;


extern (C) {
	extern const NSPoint NSZeroPoint;
	extern const NSSize NSZeroSize;
	extern const NSRect NSZeroRect;
}


NSPoint NSMakePoint(float x, float y) {
    NSPoint p;
    p.x = x;
    p.y = y;
    return p;
}

NSSize NSMakeSize(float w, float h) {
    NSSize s;
    s.width = w;
    s.height = h;
    return s;
}

NSRect NSMakeRect(float x, float y, float w, float h) {
    NSRect r;
    r.origin.x = x;
    r.origin.y = y;
    r.size.width = w;
    r.size.height = h;
    return r;
}

float NSMaxX(NSRect aRect) {
    return (aRect.origin.x + aRect.size.width);
}

float NSMaxY(NSRect aRect) {
    return (aRect.origin.y + aRect.size.height);
}

float NSMidX(NSRect aRect) {
    return (aRect.origin.x + aRect.size.width / 2.0);
}

float NSMidY(NSRect aRect) {
    return (aRect.origin.y + aRect.size.height / 2.0);
}

float NSMinX(NSRect aRect) {
    return (aRect.origin.x);
}

float NSMinY(NSRect aRect) {
    return (aRect.origin.y);
}

float NSWidth(NSRect aRect) {
    return (aRect.size.width);
}

float NSHeight(NSRect aRect) {
    return (aRect.size.height);
}

extern (C) {

	BOOL NSEqualPoints(NSPoint aPoint, NSPoint bPoint);
	BOOL NSEqualSizes(NSSize aSize, NSSize bSize);
	BOOL NSEqualRects(NSRect aRect, NSRect bRect);
	BOOL NSIsEmptyRect(NSRect aRect);

	NSRect NSInsetRect(NSRect aRect, float dX, float dY);
	NSRect NSIntegralRect(NSRect aRect);
	NSRect NSUnionRect(NSRect aRect, NSRect bRect);
	NSRect NSIntersectionRect(NSRect aRect, NSRect bRect);
	NSRect NSOffsetRect(NSRect aRect, float dX, float dY);
	void NSDivideRect(NSRect inRect, NSRect *slice, NSRect *rem, float amount, NSRectEdge edge);
	BOOL NSPointInRect(NSPoint aPoint, NSRect aRect);
	BOOL NSMouseInRect(NSPoint aPoint, NSRect aRect, BOOL flipped);
	BOOL NSContainsRect(NSRect aRect, NSRect bRect);
	BOOL NSIntersectsRect(NSRect aRect, NSRect bRect);

//	NSString *NSStringFromPoint(NSPoint aPoint);
//	NSString *NSStringFromSize(NSSize aSize);
//	NSString *NSStringFromRect(NSRect aRect);
//	NSPoint NSPointFromString(NSString *aString);
//	NSSize NSSizeFromString(NSString *aString);
//	NSRect NSRectFromString(NSString *aString);
	
}

//@interface NSValue (NSValueGeometryExtensions)
//
//+ (NSValue *)valueWithPoint:(NSPoint)point;
//+ (NSValue *)valueWithSize:(NSSize)size;
//+ (NSValue *)valueWithRect:(NSRect)rect;
//
//- (NSPoint)pointValue;
//- (NSSize)sizeValue;
//- (NSRect)rectValue;
//
//@end
//
//@interface NSCoder (NSGeometryCoding)
//
//- (void)encodePoint:(NSPoint)point;
//- (NSPoint)decodePoint;
//
//- (void)encodeSize:(NSSize)size;
//- (NSSize)decodeSize;
//
//- (void)encodeRect:(NSRect)rect;
//- (NSRect)decodeRect;
//
//@end
