
// Decagon: The D/Objective-C Bridge Demo Application
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module decagon.adder;

import cocoa.cocoa;
import objc.bridge;
import std.string;


/**
 * Controller for the Adder window which adds value from one field to the value
 * from another and shows the result in the window.
 */
class AdderController : ObjcObject {
	
	NSControl firstValueField;  /// Field for the first value.
	NSControl secondValueField; /// Field for the second value.
	NSControl resultField;      /// Field for the result of the addition.
	
	// Objective-C bindings for IB outlets.
	mixin IBOutlet!(firstValueField, "firstValueField");
	mixin IBOutlet!(secondValueField, "secondValueField");
	mixin IBOutlet!(resultField, "resultField");
	
	
	// Allow objects of this class to be created from the Objective-C side.
	// This is required for the class to be instanciated from a nib file.
	mixin ObjcBindInitializer!("init");


	/** 
	 * Calculate the sum of first value field and second value field and put 
	 * it in result field.
	 */
	void calculate(Object sender) {
		resultField.floatValue =
			firstValueField.floatValue + secondValueField.floatValue;
	}

	// Objective-C binding for IB actions.
	mixin IBAction!(calculate);

}