
// Decagon: The D/Objective-C Bridge Demo Application
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module decagon.controller;

import cocoa.cocoa;
import objc.bridge;
import std.string;


/**
 * Main controller object for our application.
 *
 * Our controller must implement the ObjcObject interface since it 
 * has Objective-C bindings.
 */
class AppController : ObjcObject {
	
	/** Counter for window title in openWindow. */
	uint windowCount;
	
	
	// Allow objects of this class to be created from the Objective-C side.
	// This is required for the class to be instanciated from a nib file.
	mixin ObjcBindInitializer!("init");


	/** Open this application's website. */
	void openWebsite(Object sender) {
		auto a = NSAlert.alert("Cannot open web site.", "OK", null, null, 
			"The D/Objective-C bridge doesn’t have access to NSWorkspace yet, "
			"so we cannot open a website. Displaying an alert works though. ;-)");
		a.runModal;
	}

	/** Open a new window and put it on the center of the screen. */
	void openWindow(Object sender) {
		auto window = new NSWindow(NSMakeRect(20, 20, 300, 200), 
			NSTitledWindowMask + NSClosableWindowMask + NSMiniaturizableWindowMask + NSResizableWindowMask,
			NSBackingStoreType.BUFFERED, false);
		window.title = "untitled "~ std.string.toString(++windowCount);
		window.center();
		window.makeKeyAndOrderFront(this);
	}

	// Objective-C binding for IB actions.
	mixin IBAction!(openWindow);
	mixin IBAction!(openWebsite);


	// Overrides from NSApplication.Delegate.

	bool openUntitledFile(NSApplication sender) {
		openWindow(sender);
		return true;
	}
	
	bool shouldTerminateAfterLastWindowClosed(NSApplication sender) {
		return true;
	}
	
	// Objective-C bindings for the above methods.
	mixin ObjcBindMethod!(openUntitledFile, void, "applicationOpenUntitledFile:", NSApplication);
	mixin ObjcBindMethod!(shouldTerminateAfterLastWindowClosed, void, "applicationShouldTerminateAfterLastWindowClosed:", NSApplication);

}