
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module objc.exceptionbridge;

import objc.runtime;


//debug = ObjcExceptionBridgeDisable;


/** Function callback for throwing ObjC exception as a D exception. */
void function(id exception) throwExceptionFunc;

/** Function callback for raising D exception as an ObjC exception. */
void function(Object exception) raiseExceptionFunc;


debug (ObjcExceptionBridgeDisable) {

	R callObjc(R)(lazy R call) { return call; }
	R callD(R)(lazy R call) { return call; }
		
} else {

	// from setjmp.h
	private extern (C) {
		// Just create a type of the correct size for jmp_buf.
		version (PPC)    private const int _JBLEN = 26 + 18*2 + 129 + 1;
		version (PPC64)  private const int _JBLEN = 26*2 + 18*2 + 129 + 1;
		version (X86)    private const int _JBLEN = 18;
		version (X86_64) private const int _JBLEN = (9 * 2) + 3 + 16;

		typedef int[_JBLEN]     jmp_buf;
		typedef int[_JBLEN + 1] sigjmp_buf;
		
		int	_setjmp(jmp_buf* env);
	}


	// from NSException.h
	private extern (C) {
		alias _setjmp _NSSETJMP;
		
		void _NSAddHandler2(NSHandler2* handler);
		void _NSRemoveHandler2(NSHandler2* handler);
		id _NSExceptionObjectFromHandler2(NSHandler2* handler);

		struct NSHandler2 {
			jmp_buf _state;
			id _exception;
			void* _others;
			void* _thread;
			void* _reserved1;
		}
	}


	/**
	 * Call Objective-C code from D, catching any exception raised and throwing them 
	 * as D exceptions using the throwExceptionFunc callback.
	 */
	R callObjc(R)(lazy R call) {
		NSHandler2 _localHandler;
		_NSAddHandler2(&_localHandler);
		if (!_NSSETJMP(&_localHandler._state)) {
		
			// Call Objective-C code that may throw exceptions here.
			static if (is(R == void)) call;
			else R result = call;

			_NSRemoveHandler2(&_localHandler);
			
			static if (is(R == void)) return;
			else return result;
		} else {
			id localException = _NSExceptionObjectFromHandler2(&_localHandler);
			
			// wrap exception in a D Object.
			throwExceptionFunc(localException);
		}
	}

	/**
	 * Call D code from Objective-C, catching any exception thrown and 
	 * raising them as Objective-C exceptions using the raiseExceptionFunc
	 * callback.
	 */
	R callD(R)(lazy R call) {
		try {
		
			// Call D code that may throw an exception.
			return call;
			
		} catch (Object o) {
			raiseExceptionFunc(o);
		}
	}

}
