
// The D/Objective-C Bridge
// Copyright © 2007  Michel Fortin
// <http://michelf.com/d-objc-bridge/>
//
// This program is free software: you can redistribute it and/or modify it 
// under the terms of the GNU General Public License as published by the Free 
// Software Foundation, either version 3 of the License, or (at your option) 
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT 
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
// more details.
//
// You should have received a copy of the GNU General Public License along 
// with this program. If not, see <http://www.gnu.org/licenses/>.

module objc.objectlookup;

import objc.runtime;


version = ObjectLookupMangledPtr;

private version (ObjectLookupMangledPtr) {

	/** Mangled pointer type to avoid the garbage collector notifying the pointers. */
	alias size_t ObjectRefType;

	size_t mangle(Object obj)   { return ~cast(size_t)obj; }
	Object unmangle(size_t obj) { return cast(Object)~obj; }

} else {

	alias Object ObjectRefType;

	Object mangle(Object obj)   { return obj; }
	Object unmangle(Object obj) { return obj; }

}

/** Object registration. */
private ObjectRefType[id] objectLookup;


/**
 * Lookup for object in 
 */
Object lookup(id instance) {
	ObjectRefType* ptr = (instance in objectLookup);
	if (ptr) return unmangle(*ptr);
	else     return null;
}


/**
 * Register object in lookup table.
 */
public void register(id instance, Object object) {
	objectLookup[instance] = mangle(object);
}


/**
 * Unregister object from lookup table.
 */
public void unregister(id instance)  {
	objectLookup.remove(instance);
}

